package flutter.dsw.com.flutter_plugin_sensors;

import android.content.Context;
import android.text.TextUtils;

import com.sensorsdata.analytics.android.sdk.ISensorsDataAPI;
import com.sensorsdata.analytics.android.sdk.SALog;
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import io.flutter.plugin.common.MethodCall;

public class SensorsDataAPIHelp {
    private static ISensorsDataAPI sensorsDataAPI;

    private SensorsDataAPIHelp(Context context) {
        if (sensorsDataAPI == null) {
            sensorsDataAPI = SensorsDataAPI.sharedInstance(context);
        }
    }

    public static SensorsDataAPIHelp getInstance(Context context) {
        return new SensorsDataAPIHelp(context);
    }

    /**
     * 首次启动安装
     *
     * @param methodCall MethodCall
     */
    void trackInstallation(MethodCall methodCall) {
        try {
            final String EVENT_NAME = "eventName";
            final String PROPERTY = "property";
            final String CALL_BACK = "property";
            if (methodCall.hasArgument(EVENT_NAME)) {
                String eventName = methodCall.argument(EVENT_NAME);
                Map<String, Object> mapProperty = null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProperty = methodCall.argument(PROPERTY);
                }

                boolean isCallBack = false;
                if (methodCall.hasArgument(CALL_BACK)) {
                    try {
                        isCallBack = methodCall.argument(CALL_BACK);
                    } catch (Exception ex) {
                        SALog.printStackTrace(ex);
                    }
                }

                SensorsDataAPI.sharedInstance().trackInstallation(eventName, mapToJson(mapProperty),
                        isCallBack);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 登录，设置当前用户的 loginId
     * @param methodCall MethodCall
     */
    void login(MethodCall methodCall) {
        try {
            final String LOGIN_ID = "loginId";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(LOGIN_ID)) {
                String loginId = methodCall.argument(LOGIN_ID);
                Map<String, Object> mapProperty = null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProperty = methodCall.argument(PROPERTY);
                }
                SensorsDataAPI.sharedInstance().login(loginId, mapToJson(mapProperty));
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 注销，清空当前用户的 loginId
     */
    void logout() {
        try {
            SensorsDataAPI.sharedInstance().logout();
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 调用 track 接口，追踪一个带有属性的事件
     *
     * @param methodCall MethodCall
     */
    void track(MethodCall methodCall) {
        try {
            final String EVENT_NAME = "eventName";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(EVENT_NAME)) {
                String eventName = methodCall.argument(EVENT_NAME);
                Map<String, Object> mapProperty = null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProperty = methodCall.argument(PROPERTY);
                }

                SensorsDataAPI.sharedInstance().track(eventName, mapToJson(mapProperty));
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 给一个列表类型的 Profile 增加一个元素
     *
     * @param methodCall MethodCall
     */
    void profileAppend(MethodCall methodCall) {
        try {
            final String VALUE = "value";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                String value = null;
                if (methodCall.hasArgument(VALUE)) {
                    value = methodCall.argument(VALUE);
                }

                SensorsDataAPI.sharedInstance().profileAppend(property, value);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 给一个列表类型的 Profile 增加一个元素
     *
     * @param methodCall MethodCall
     */
    void profileAppendValues(MethodCall methodCall) {
        try {
            final String VALUE = "value";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                List<String> values = null;
                if (methodCall.hasArgument(VALUE)) {
                    values = methodCall.argument(VALUE);
                }

                SensorsDataAPI.sharedInstance().profileAppend(property, new HashSet<>(values));
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 删除用户的一个 Profile
     *
     * @param methodCall MethodCall
     */
    void profileUnset(MethodCall methodCall) {
        try {
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                SensorsDataAPI.sharedInstance().profileUnset(property);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 删除用户所有 Profile
     */
    void profileDelete() {
        try {
           SensorsDataAPI.sharedInstance().profileDelete();
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 设置用户的一个 Profile, Profile如果存在，则覆盖；否则，新创建。
     *
     * @param methodCall MethodCall
     */
    void profileSet(MethodCall methodCall) {
        try {
            final String VALUE = "value";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                String value = null;
                if (methodCall.hasArgument(VALUE)) {
                    value = methodCall.argument(VALUE);
                }

                SensorsDataAPI.sharedInstance().profileSet(property, value);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 设置用户的一个或多个 Profile, Profile如果存在，则覆盖；否则，新创建。
     *
     * @param methodCall MethodCall
     */
    void profileSetProperties(MethodCall methodCall) {
        try {
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                Map<String, Object> mapProfile= null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProfile = methodCall.argument(PROPERTY);
                }

                SensorsDataAPI.sharedInstance().profileSet(mapToJson(mapProfile));
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 设置用户的一个 Profile, Profile如果存在，则覆盖；否则，新创建。
     *
     * @param methodCall MethodCall
     */
    void profileSetOnce(MethodCall methodCall) {
        try {
            final String VALUE = "value";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                String value = null;
                if (methodCall.hasArgument(VALUE)) {
                    value = methodCall.argument(VALUE);
                }

                SensorsDataAPI.sharedInstance().profileSetOnce(property, value);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 设置用户的一个或多个 Profile, Profile如果存在，则覆盖；否则，新创建。
     *
     * @param methodCall MethodCall
     */
    void profileSetOnceProperties(MethodCall methodCall) {
        try {
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                Map<String, Object> mapProfile= null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProfile = methodCall.argument(PROPERTY);
                }

                SensorsDataAPI.sharedInstance().profileSetOnce(mapToJson(mapProfile));
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 给一个数值类型的 Profile 增加一个数值。只能对数值型属性进行操作，若该属性
     * 未设置，则添加属性并设置默认值为 0
     *
     * @param methodCall MethodCall
     */
    void profileIncrement(MethodCall methodCall) {
        try {
            final String VALUE = "value";
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                String property = methodCall.argument(PROPERTY);
                int value = 0;
                if (methodCall.hasArgument(VALUE)) {
                    value = methodCall.argument(VALUE);
                }

                SensorsDataAPI.sharedInstance().profileIncrement(property, value);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    /**
     * 给一个或多个数值类型的 Profile 增加一个数值。只能对数值型属性进行操作，若该属性
     * 未设置，则添加属性并设置默认值为 0
     *
     * @param methodCall MethodCall
     */
    void profileIncrementProperties(MethodCall methodCall) {
        try {
            final String PROPERTY = "property";
            if (methodCall.hasArgument(PROPERTY)) {
                Map<String, ? extends Number> mapProfile = null;
                if (methodCall.hasArgument(PROPERTY)) {
                    mapProfile = methodCall.argument(PROPERTY);
                }

                SensorsDataAPI.sharedInstance().profileIncrement(mapProfile);
            }
        } catch (Exception ex) {
            SALog.printStackTrace(ex);
        }
    }

    private JSONObject mapToJson(Map<String, Object> map){
        if (map == null) {
            return null;
        }
        JSONObject jsonObject = new JSONObject();
        for (String key: map.keySet()){
            try {
                if (TextUtils.isEmpty(key)){
                    return null;
                }
                jsonObject.put(key, map.get(key));
            } catch (JSONException e) {
                SALog.printStackTrace(e);
            }
        }

        return jsonObject;
    }
}
