import 'dart:async';

import 'package:flutter/services.dart';

class FlutterPluginSensors {
  static const MethodChannel _channel =
      const MethodChannel('flutter_plugin_sensors');

  static const String TRACK_INSTALLATION = "trackInstallation";
  static const String LOGIN = "login";
  static const String LOGOUT = "logout";
  static const String TRACK = "track";
  static const String PROFILE_APPEND = "profileAppend";
  static const String PROFILE_APPEND_VALUES = "profileAppendValues";
  static const String PROFILE_UNSET = "profileUnset";
  static const String PROFILE_DELETE = "profileDelete";
  static const String PROFILE_SET = "profileSet";
  static const String PROFILE_SET_PROPERTIES = "profileSetProperties";
  static const String PROFILE_SET_ONCE = "profileSetOnce";
  static const String PROFILE_SET_ONCE_PROPERTIES = "profileSetOnceProperties";
  static const String PROFILE_INCREMENT = "profileIncrement";
  static const String PROFILE_INCREMENT_PROPERTIES = "profileIncrementProperties";

  /// 首次启动安装
  static Future<Null> trackInstallation(String eventName,
              [Map<String, Object> mapProperty, bool disableCallback]) async {
    return await _channel.invokeMethod(TRACK_INSTALLATION,
        {"eventName": eventName, "property": mapProperty, "callback": disableCallback});
  }

  /// 登录，设置当前用户的 loginId
  static Future<Null> login(String loginId, [Map<String, Object> mapProperty]) async {
    return await _channel.invokeMethod(LOGIN, {"loginId": loginId, "property": mapProperty});
  }

  /// 注销，清空当前用户的 loginId
  static Future<Null> logout() async{
    return await _channel.invokeMethod(LOGOUT);
  }

  /// 调用 track 接口，追踪一个带有属性的事件
  static Future<Null> track(String eventName, [Map<String, Object> mapProperty]) async {
    return await _channel.invokeMethod(TRACK, {"eventName": eventName, "property": mapProperty});
  }

  /// 给一个列表类型的 Profile 增加一个元素
  static Future<Null> profileAppend(String property, String value) async {
    return await _channel.invokeMethod(PROFILE_APPEND, {"property": property, "value": value});
  }

  /// 给一个列表类型的 Profile 增加一个或多个元素
  static Future<Null> profileAppendValues(String property, List<String> values) async {
    return await _channel.invokeMethod(PROFILE_APPEND_VALUES, {"property": property, "value": values});
  }

  /// 删除用户的一个 Profile
  static Future<Null> profileUnset(String property) async {
    return await _channel.invokeMethod(PROFILE_UNSET, {"property": property});
  }

  /// 删除用户的一个 Profile
  static Future<Null> profileDelete() async {
    return await _channel.invokeMethod(PROFILE_DELETE);
  }

  /// 设置用户的一个 Profile，如果之前存在，则覆盖，否则，新创建
  static Future<Null> profileSet(String property, String value) async {
    return await _channel.invokeMethod(PROFILE_SET, {"property": property, "value": value});
  }

  /// 设置用户的一个或多个 Profile，如果之前存在，则覆盖，否则，新创建
  static Future<Null> profileSetProperties(Map<String, String> mapProfile) async {
    return await _channel.invokeMethod(PROFILE_SET_PROPERTIES, {"property": mapProfile});
  }

  /// 首次设置用户的一个 Profile。 与 profileSet 接口不同的是，如果之前存在，则忽略，否则，新创建
  static Future<Null> profileSetOnce(String property, String value) async {
    return await _channel.invokeMethod(PROFILE_SET, {"property": property, "value": value});
  }

  /// 首次设置用户的一个或多个 Profile。 与 profileSet 接口不同的是，如果之前存在，则忽略，否则，新创建
  static Future<Null> profileSetOnceProperties(Map<String, String> mapProfile) async {
    return await _channel.invokeMethod(PROFILE_SET_PROPERTIES, {"property": mapProfile});
  }

  /// 给一个数值类型的 Profile 增加一个数值。只能对数值型属性进行操作，若该属性
  /// 未设置，则添加属性并设置默认值为 0
  static Future<Null> profileIncrement(String property, int value) async {
    return await _channel.invokeMethod(PROFILE_INCREMENT, {"property": property, "value": value});
  }

  /// 给一个或多个数值类型的 Profile 增加一个数值。只能对数值型属性进行操作，若该属性
  /// 未设置，则添加属性并设置默认值为 0
  static Future<Null> profileIncrementProperties(Map<String, num> mapProfile) async {
    return await _channel.invokeMethod(PROFILE_INCREMENT_PROPERTIES, {"property": mapProfile});
  }
}
